'*********************************************
'* Spin code generated by cspin version 0.82 *
'*********************************************
{
 *   Copyright (c) 2001, Carlos E. Vidales. All rights reserved.
 *    This sample program was written and put in the public domain by Carlos E. Vidales.  The program is provided "as is" without warranty of any kind,
 *    either expressed or implied. If you choose to use the program within your own products you do so at your own risk, and assume the responsibility
 *    for servicing, repairing or correcting the program should it prove defective in any manner. You may copy and distribute the program's source code
 *    in any  medium, provided that you also include in each copy an appropriate copyright notice and disclaimer of warranty. You may also modify this
 *    program and distribute copies of it provided that you include prominent notices stating that you changed the file(s) and the date of any change,
 *    and that you do not charge any royalties or licenses for its use.

  NOTE: Even though the validity of the calibration/translation method proposed has been verified with empirical data from several actual touch screen
  enabled displays, for the convenience of this exercise, the raw and expected data used and presented below are artificial.  When used with actual data
  the functions presented yield results that may be off by a larger but still small percentage (~1-3%) due to electrical noise and human error (i.e., the
  hand touching a screen target and missing by a small amount.) The array of input points.  The first three are used for calibration. These set of points
  assume that the touchscreen has vertical and horizontal resolutions of 1024 pixels (10-bit digitizer.)

                NOTE!    NOTE!    NOTE!                                                              
  setCalibrationMatrix() and getDisplayPoint() will do fine for you as they are, provided that your digitizer resolution does not exceed 10 bits
 (1024 values). Higher resolutions may cause the integer operations to overflow and return incorrect values. If you wish to use these functions with
 digitizer resolutions of 12 bits (4096 values) you will either have to a) use 64-bit signed integer variables and math, or b) judiciously modify the
 operations to scale results by a factor of 2 or even 4.
 
 *  From the article text, recall that the matrix coefficients are resolved to be the following:
 *
                                   185_697 + 297_515 = 233_549_275
                         -439 * -423       -        379 *  -785
                   (073 - 512)*(516 - 939) - (891 - 512)*(154 - 939)  
 *      Divider =  (Xs0 - Xs2)*(Ys1 - Ys2) - (Xs1 - Xs2)*(Ys0 - Ys2)
 *
 *
 *
 *                 (Xd0 - Xd2)*(Ys1 - Ys2) - (Xd1 - Xd2)*(Ys0 - Ys2)
 *            A = ---------------------------------------------------
 *                                   Divider
 *
 *
 *                 (Xs0 - Xs2)*(Xd1 - Xd2) - (Xd0 - Xd2)*(Xs1 - Xs2)
 *            B = ---------------------------------------------------
 *                                   Divider
 *
 *
 *                 Ys0*(Xs2*Xd1 - Xs1*Xd2) + 
 *                             Ys1*(Xs0*Xd2 - Xs2*Xd0) + 
 *                                           Ys2*(Xs1*Xd0 - Xs0*Xd1)
 *            C = ---------------------------------------------------
 *                                   Divider
 *
 *
 *                 (Yd0 - Yd2)*(Ys1 - Ys2) - (Yd1 - Yd2)*(Ys0 - Ys2)
 *            D = ---------------------------------------------------
 *                                   Divider
 *
 *
 *                 (Xs0 - Xs2)*(Yd1 - Yd2) - (Yd0 - Yd2)*(Xs1 - Xs2)
 *            E = ---------------------------------------------------
 *                                   Divider
 *
 *
 *                 Ys0*(Xs2*Yd1 - Xs1*Yd2) + 
 *                             Ys1*(Xs0*Yd2 - Xs2*Yd0) + 
 *                                           Ys2*(Xs1*Yd0 - Xs0*Yd1)
 *            F = ---------------------------------------------------
 *                                   Divider
 *
 }
CON                                              
  _clkmode      = xtal1 + pll16x                        ' use crystal x 16
  _xinfreq      = 6_250_000                             ' 5Mhz crystal


VAR
  byte font  

OBJ
        cal : "cal.spin"
        tch : "Touch_v1_0_1963a.spin"

DAT      
{{
  The array of input points. The first three are used for calibration. These set of points assume that the touchscreen has vertical and horizontal
  resolutions of 1024 pixels (10-bit digitizer.)
}}
  screenXSample long 073, 891, 512
  screenYSample long 154, 516, 939
{{
  The array of expected "right answers." The values selected assume a vertical and horizontal display resolution of 240 pixels.(Actual display is 480*800)
}}           
  displayXSample long 060, 420, 240
  displayYSample long 100, 400, 700
{{
  An array of perfect input screen points used to obtain a first pass calibration matrix good enough to collect calibration samples.
}}
  perfectXScreenSample long  100, 900, 500
  perfectYScreenSample long  100, 500, 900
{{
  An array of perfect display points used to obtain a first pass calibration matrix good enough to collect calibration samples.
}}
  perfectXDisplaySample long 100, 900, 500
  perfectYDisplaySample long 100, 500, 900  

PUB main |  n, m, x, y 
{*********************************************************************
 *  Function   : main()
 *  Description: Entry point into console version of sample
 *                program that exercises the calibration 
 *                functions. 
 *  Argument(s): argCount - the number of arguments provided
 *               argValue - pointer to the list of char strings 
 *                           representing the command line arguments. 
}

  tch.BeginDesktop("7")               
  font := tch.LoadFont(string("sysfont.ifn"))
  tch.Clearscreen($FFFe)  
  tch.SetCursor(0,0)
  tch.TextT(font, string("Touch Screen Calibration Sample Code,",13,10))
  tch.TextT(font, string("by Carlos E. Vidales, Copyright 2001.",13,10))
  tch.TextT(font, string(13,10))
        
{{
  The following call calculates the translation matrix that results when the three consecutive points in the sample set are used. Such points are assumed
  to be properly spaced within the screen surface. Note that we call the function twice as we would normally do within a calibration routine. The first
  time we call it using a perfect set of display and screen arguments. Such a call is made to obtain a calibration matrix that is just good enough to
  collect samples to do the real calibration.
  Look at the matrix values when we use a perfect sample set. The result is a unity matrix.
}}

  if cal.setCalibrationMatrix(@perfectXDisplaySample, @perfectYDisplaySample, @perfectXScreenSample, @perfectYScreenSample)
    n := cal.getMatrixAddress                                                                                              
    tch.TextT(font,string(13,10,"Look at the unity matrix:",13,10,13,10))
    tch.TextT(font,string("An=             Bn=             Cn=      ",13,10 ))
    tch.TextT(font,string("Dn=             En=             Fn=      ",13,10))
    tch.TextT(font,string("          Divider =      ",13,10))
    tch.SetCursor(38,120)  
    tch.TextDec(font,long[n][0])
    tch.SetCursor(200,120)
    tch.TextDec(font,long[n][1])
    tch.SetCursor(360,120)
    tch.TextDec(font,long[n][2])
    tch.SetCursor(38,140)  
    tch.TextDec(font,long[n][3])
    tch.SetCursor(200,140)  
    tch.TextDec(font,long[n][4])
    tch.SetCursor(360,140)  
    tch.TextDec(font,long[n][5])
    tch.SetCursor(200,160)  
    tch.TextDec(font,long[n][6]) 
  else
    tch.TextT(font,string(13,10,"Calibration Failed",13,10,13,10))
    
{{
  Now is when we need to do the work to collect a real set of calibration data. Draw three targets on your display. Drawing one at time is probably a
  simpler implementation. These targets should be widely separated but also avoid the areas too near the edges where digitizer output tends to become
  non-linear. The recommended set of points is (in display resolution percentages):       
                     ( 15, 15)                                    
                     ( 50, 85)                                 
                     ( 85, 50)                                                                
  Each time save the display and screen set (returned by the digitizer when the user touches each calibration target into the corresponding array).
  Since you normalized your calibration matrix above, you should be able to use touch screen data as it would be provided by the digitizer driver.
  When the matrix equals unity, getDisplayPoint() returns the same raw input data as output. Call the function once more to obtain the calibration factors
  you will use until you calibrate again.
  Let's see the matrix values for no particular reason.
}}
  
  if cal.setCalibrationMatrix(@displayXSample, @displayYSample, @screenXSample, @screenYSample)     
    n := cal.getMatrixAddress
    tch.SelectMemGroup 
    tch.TextT(font,string(13,10,13,10,"This is the actual calibration matrix that we",13,10))
    tch.TextT(font,string("will use for all points",13,10))
    tch.TextT(font,string("(until we calibrate again):",13,10))
    tch.TextT(font,string("An=             Bn=             Cn=      ",13,10 ))
    tch.TextT(font,string("Dn=             En=             Fn=      ",13,10))
    tch.TextT(font,string("          Divider =      ",13,10))
    tch.SetCursor(38,260)  
    tch.TextDec(font,long[n][0])
    tch.SetCursor(200,260)
    tch.TextDec(font,long[n][1])
    tch.SetCursor(360,260)
    tch.TextDec(font,long[n][2])
    tch.SetCursor(38,280)  
    tch.TextDec(font,long[n][3])
    tch.SetCursor(200,280)  
    tch.TextDec(font,long[n][4])
    tch.SetCursor(360,280)  
    tch.TextDec(font,long[n][5])
    tch.SetCursor(200,300)  
    tch.TextDec(font,long[n][6])
  else
    tch.TextT(font,string(13,10,"Calibration Failed",13,10,13,10))

{{
  Now, lets use the complete set of screen samples to verify that the calculated calibration matrix does its job as expected.
  In a real application, your digitizer driver interrupt would probably do the following:
         1) collect raw digitizer data,                         
         2) filter the raw data which would probably contain position jitter                                                
         3) filter out repeated values (a touch screen controller normally continues causing interrupts and collecting data as long as the user is pressing
            on the screen), and                                
         4) call the function getDisplayPoint() to obtain the display coordinates that the user meant to input as he touched the screen.                                                                          
  This code sample, of course, only uses sample data. So we simply run through all the available sample points
}}

  tch.SelectMemGroup
  tch.TextT(font,string(13,10,13,10,"Show the results of our work:",13,10,13,10))
  tch.TextT(font,string("Screen Sample  Translated Sample  Display Sample"))

  n := cal.getMatrixAddress
  repeat m from 0 to cal#MAX_SAMPLES - 1
    tch.TextT(font,string(13,10,13,10)) 
    tch.TextT(font,string("   X =  ")) 
    tch.TextDec(font,long[@screenXSample][m])
    tch.TextT(font,string("          "))
    tch.TextDec(font,cal.getXDisplayPoint(long[@screenXSample][m],long[@screenYSample][m]))
    tch.TextT(font,string("          "))
    tch.TextDec(font,long[@displayXSample][m])

    tch.TextT(font,string(13,10))
    tch.TextT(font,string("   Y =  ")) 
    tch.TextDec(font,long[@screenYSample][m])
    tch.TextT(font,string("          ")) 
    tch.TextDec(font,cal.getYDisplayPoint(long[@screenXSample][m], long[@screenYSample][m]))
    tch.TextT(font,string("          "))  
    tch.TextDec(font,long[@displayYSample][m])

  tch.TextT(font,string(13,10,13,10,13,10,13,10,"          Touch Screen to continue")) 
  repeat while tch.TouchValue  == $FFFF

  '' now we do an actual calibration

  tch.clearscreen($FFFe)                                                   
  tch.SetCursor(0,0)
  tch.TextT(font,string(13,10,"Now lets try an actual calibration",13,10))
  tch.TextT(font,string(13,10,"First reset the matrix to unity values",13,10))
  
    if cal.setCalibrationMatrix(@perfectXDisplaySample, @perfectYDisplaySample, @perfectXScreenSample, @perfectYScreenSample)
      n := cal.getMatrixAddress
      tch.TextT(font,string("Look at the unity matrix:",13,10,13,10))
      tch.TextT(font,string("An=             Bn=             Cn=      ",13,10 ))
      tch.TextT(font,string("Dn=             En=             Fn=      ",13,10))
      tch.TextT(font,string("          Divider =      ",13,10))
      tch.SetCursor(38,120)  
      tch.TextDec(font,long[n][0])
      tch.SetCursor(200,120)
      tch.TextDec(font,long[n][1])
      tch.SetCursor(360,120)
      tch.TextDec(font,long[n][2])
      tch.SetCursor(38,140)  
      tch.TextDec(font,long[n][3])
      tch.SetCursor(200,140)  
      tch.TextDec(font,long[n][4])
      tch.SetCursor(360,140)  
      tch.TextDec(font,long[n][5])
      tch.SetCursor(200,160)  
      tch.TextDec(font,long[n][6]) 
    else
      tch.TextT(font,string(13,10,"Calibration Failed",13,10,13,10))

  tch.TextT(font,string(13,10,13,10,13,10,13,10,"          Touch the X on the next screens")) 
  tch.TextT(font,string(13,10,13,10,13,10,13,10,"          Touch Screen to continue")) 
  repeat while tch.TouchValue  == $FFFF

  repeat n from 0 to cal#MAX_SAMPLES - 1
    tch.SetCursor(long[@displayXSample][n],long[@displayYSample][n])
    tch.SelectMemGroup
    tch.clearscreen($FFFe)   
    tch.TextT(font,string("X"))
      repeat while (result := tch.TouchValue) == $FFFF
      x := result & $FFFF
      y := (result >> 16) & $FFFF
        repeat 1_000
          repeat while (result := tch.TouchValue) == $FFFF
          x += result & $FFFF
          y += (result >> 16) & $FFFF
          x /= 2
          y /= 2
    long[@screenXSample][n] := x
    long[@screenYSample][n] := y        
    tch.clearscreen($FFFe)
    tch.SetCursor(200,400)
    tch.TextT(font,string("DONE"))
    tch.pause1ms(2000)

  tch.SetCursor(0,0)
  if cal.setCalibrationMatrix(@displayXSample, @displayYSample, @screenXSample, @screenYSample)     
    n := cal.getMatrixAddress
    tch.SelectMemGroup
    tch.clearscreen($FFFe)    
    tch.TextT(font,string(13,10,13,10,"  This is the actual calibration matrix",13,10))
    tch.TextT(font,string("  from the points we collected",13,10))
    tch.TextT(font,string(13,10,13,10))
    tch.TextT(font,string("An=             Bn=             Cn=      ",13,10 ))
    tch.TextT(font,string("Dn=             En=             Fn=      ",13,10))
    tch.TextT(font,string("          Divider =      ",13,10))
    tch.SetCursor(38,120)  
    tch.TextDec(font,long[n][0])
    tch.SetCursor(200,120)
    tch.TextDec(font,long[n][1])
    tch.SetCursor(360,120)
    tch.TextDec(font,long[n][2])
    tch.SetCursor(38,140)  
    tch.TextDec(font,long[n][3])
    tch.SetCursor(200,140)  
    tch.TextDec(font,long[n][4])
    tch.SetCursor(360,140)  
    tch.TextDec(font,long[n][5])
    tch.SetCursor(200,160)  
    tch.TextDec(font,long[n][6])
  else
    tch.TextT(font,string(13,10,"Calibration Failed",13,10,13,10))
    
  tch.TextT(font,string(13,10,13,10,13,10,13,10,"          Touch Screen to continue"))           
  repeat while tch.TouchValue  == $FFFF
  
  tch.SelectMemGroup
  tch.clearscreen($FFFe)                                                   
  tch.SetCursor(0,0)                    
  tch.TextT(font,string(13,10,13,10,"Show the results of our work:",13,10,13,10))
  tch.TextT(font,string("Screen Sample  Translated Sample  Display Sample"))

  n := cal.getMatrixAddress
  repeat m from 0 to cal#MAX_SAMPLES - 1

    tch.TextT(font,string(13,10,13,10)) 
    tch.TextT(font,string("   X =  ")) 
    tch.TextDec(font,long[@screenXSample][m])
    tch.TextT(font,string("          "))
    tch.TextDec(font,cal.getXDisplayPoint(long[@screenXSample][m],long[@screenYSample][m]))
    tch.TextT(font,string("          "))
    tch.TextDec(font,long[@displayXSample][m])

    tch.TextT(font,string(13,10))
    tch.TextT(font,string("   Y =  ")) 
    tch.TextDec(font,long[@screenYSample][m])
    tch.TextT(font,string("          ")) 
    tch.TextDec(font,cal.getYDisplayPoint(long[@screenXSample][m], long[@screenYSample][m]))
    tch.TextT(font,string("          "))  
    tch.TextDec(font,long[@displayYSample][m])

  tch.TextT(font,string(13,10,13,10,13,10,13,10,"          Touch Screen to continue")) 
  repeat while tch.TouchValue  == $FFFF

  repeat 
    result := tch.TouchValue
    ifnot result == $FFFF
      x := result & $FFFF
      y := (result >> 16)& $FFFF
      tch.clearscreen(0)
      tch.ClearRectangle(x-10,y-10,x+10,y+10,$FFFe)
      tch.pause1ms(500) 
  


'*********************************************
'* Spin code generated by cspin version 0.82 *
'*********************************************
{
 *   Copyright (c) 2001, Carlos E. Vidales. All rights reserved.
 *    This sample program was written and put in the public domain by Carlos E. Vidales.  The program is provided "as is" without warranty of any kind,
 *    either expressed or implied. If you choose to use the program within your own products you do so at your own risk, and assume the responsibility
 *    for servicing, repairing or correcting the program should it prove defective in any manner. You may copy and distribute the program's source code
 *    in any  medium, provided that you also include in each copy an appropriate copyright notice and disclaimer of warranty. You may also modify this
 *    program and distribute copies of it provided that you include prominent notices stating that you changed the file(s) and the date of any change,
 *    and that you do not charge any royalties or licenses for its use.

  NOTE: Even though the validity of the calibration/translation method proposed has been verified with empirical data from several actual touch screen
  enabled displays, for the convenience of this exercise, the raw and expected data used and presented below are artificial.  When used with actual data
  the functions presented yield results that may be off by a larger but still small percentage (~1-3%) due to electrical noise and human error (i.e., the
  hand touching a screen target and missing by a small amount.) The array of input points.  The first three are used for calibration. These set of points
  assume that the touchscreen has vertical and horizontal resolutions of 1024 pixels (10-bit digitizer.)
 }